%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% ST_CH.M

% This file contains the Matlab procedure ST_CH

%     written by

% Bruce E. Hansen
% Department of Economics
% Social Science Building
% University of Wisconsin
% Madison, WI 53706-1393
% bhansen@ssc.wisc.edu
% http://www.ssc.wisc.edu/~bhansen/

% Note: the procedure calls the procedures
% pv_sup, pv_exp, and pv_ave
% which are contained in separate ASCII files.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% ST_CH
% Procedure to calculate test statistics and p-values for linear
% regression models subject to structural change.

% Format:

% [kest,sup,exp,ave] = st_ch(y,x,t1,t2,h);

% Inputs:
% y = dependent variable (nx1 vector).
% x = indenpendent variables (nxk matrix).
% t1 = starting breakpoint percentage, number in (0,1)
%      or starting breakpoint index, number in [k,T-k].
% t2 = ending breakpoint percentage, number in (0,1)
%      or ending breakpoint index, number in [k,T-k].
% h  = Heteroskedasticity-Consistent Covariance Matrix dummy
%      1 to calculate heteroskedasticity-consistent covariances
%      0 to calculate conventional covariances.

% Outputs:
% kest = Quandt estimate of breakdate
% sup  = Andrews/Quandt SupLM test
% exp  = Andrews/Ploberger ExpLM test (c=infinity)
% ave  = Andrews/Ploberger AveLM test (c=0)

% kest is a (k+1)x2 vector.
%   Estimates of Breakpoint.
%   The first row reports the index of the element (i.e., the 22nd observation),
%     while the second reports the index's relative place in the sample
%     (i.e., if n=200, it equals 22/200=.11).
%  The first element is the estimate of breakpoint obtained by allowing all
%     parameter to shift.  The remaining k elements contain the estimates of
%     the breakpoint when only a single parameter is allowed to shift.

% Each of (sup, exp, ave) are (k+1)x2 matrices.
%   The first column contains test statistics, the second contains
%     asymptotic p-values calcuated from by the procedures pv_sup,
%     pv_exp, and pv_ave.
%   The first row reports the joint test for constancy of all regression
%     parameters.
%   The remaining rows report the tests for partial constancy of each
%     regression parameter, separately.

% Note:
% For Sup test, Andrews (1993) recommends t1 = .15 and t2=.85.
% For Exp and Ave tests, Andrews-Ploberger (1994) recommend t1 = .02 and t2=.98.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [kest,suptpsup,exptpexp,avetpave]=st_ch(y,x,t1,t2,h);

xx=x'*x;
xxi=inv(xx);
e=y-x*xxi*(x'*y);
k=length(x(1,:));
n=length(e(:,1));
sig=(e'*e)/(n-k);
if h==1;
    xe=x.*(e*ones(1,length(x(1,:))));
else
    xe=x*sqrt(sig);
end;
xes=cumsum(x.*(e*ones(1,length(x(1,:)))));
mgm=xxi*(xe'*xe)*xxi;
if t1<1
    n1=floor(n*t1);
    n2=floor(n*t2);
    tau1=t1;
    tau2=t2;
else
    n1=t1;
    n2=t2;
    tau1=t1/n;
    tau2=t2/n;
end;

if n1<k
    disp('');disp('');
    disp('ERROR:  Ending Sample is smaller than Number of Paramters');
    disp('        You need to select a smaller value for t2');
    disp('');disp('');
end;

if n2>(n-k)
    disp('');disp('');
    disp('ERROR:  Ending Sample is smaller than Number of Paramters');
    disp('        You need to select a smaller value for t2');
    disp('');disp('');
end;

lm=zeros(n,k+1);
m=x(1:n1-1,:)'*x(1:n1-1,:);
g=xe(1:n1-1,:)'*xe(1:n1-1,:);

i=n1;
while i<=n2
    m=m+x(i,:)'*x(i,:);
    g=g+xe(i,:)'*xe(i,:);
    mi=g-m*xxi*g-g*xxi*m+m*mgm*m;
    sn=xes(i,:)';
    pw=(sn.^2)./diag(mi);
    jw=sn'*inv(mi)*sn;
    lm(i,:)=[jw,pw'];
    i=i+1;
end;

[supt,kest]=max(lm);
supt=supt';
kest=[kest,kest/n];
lmt=lm(n1:n2,:);
expt=log(mean(exp(lmt/2))');
avet=mean(lmt)';

pi0=1/(1+sqrt(tau2*(1-tau1)/tau1/(1-tau2)));

psup=pv_sup(supt(1),k,pi0);
pexp=pv_exp(expt(1),k,pi0);
pave=pv_ave(avet(1),k,pi0);

i=1;
while i<=k
    psup = [psup;pv_sup(supt(i+1),1,pi0)];
    pexp = [pexp;pv_exp(expt(i+1),1,pi0)];
    pave = [pave;pv_ave(avet(i+1),1,pi0)];
    i=i+1;
end;
suptpsup=[supt,psup];
exptpexp=[expt,pexp];
avetpave=[avet,pave];
    
    
    